local mod = MorgDKP2:NewModule("BidWar", "AceComm-3.0", "AceSerializer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")

local mdkp = MorgDKP2

mod.modName = L["BidWar"]
mod.modref = "BidWar"

local Player = UnitName("player")
local db
local fmt = string.format

local defaults = { 
	profile = {
		bidstep = 5,
		overbid = 50,
		silent = nil,
		justenough = nil
	},
}

function mod:GetOptions()
	return options
end

function mod:GetName()
	return mod.modName, mod.modref
end

function mod:IsDKP()
	return true
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("BidWar", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	db.moduleON["BidWar"] = true
	self.MaxBid = {}
	self.BidWarDone = nil
	self.master = {}
	self.hasbid = {}
	self:CheckConflicts()
end

function mod:OnDisable()
	db.moduleON["BidWar"] = nil
	local currdkp = self:GetModuleRef(db.currDKP)
	if currdkp then db.dkpstatustext = currdkp:GetName()
	else db.dkpstatustext = L["Not Set"] end
	self:ToggleFrameUpdate("TooltipLDB")
end

function mod:Toggle(info, v)
	mdkp.db.profile.modules.BidWar = v
	if v then mod:Enable()
	else mod:Disable() end
end

function mod:SetOpt(info, v)
	local db = mod.db.profile
	local opt = info[#info]
	db[opt] = v
end

function mod:GetOpt(info)
	local db = mod.db.profile
	local opt = info[#info]
	return db[opt]
end

function mod:UpdateDB()
	db = mdkp.db.profile
end

function mod:CheckConflicts()
	local currconflicts = mdkp.bidwarconflicts
	local isconflict
	for _, conflictname in pairs(currconflicts) do
		if db.modules[conflictname] then 
			local conflict = MorgDKP2:GetModule(conflictname)
			conflict:Disable()
			db.modules[conflictname] = nil
			self:debug("Disabled module.."..conflictname.." due to conflict")
			isconflict = true
			db.currDKP = nil
			break
		end
	end
	if isconflict then
		db.currDKP = nil
		self:IsDKPEnabled()
		db.dkpstatustext = "Fixed DKP & BidWar"
	else 
		if not string.find(db.dkpstatustext, "&") then db.dkpstatustext = db.dkpstatustext .. " & " .. L["BidWar"] end
	end
	self:ToggleFrameUpdate("TooltipLDB")
end

function mod:StartAuction(id, itempoints, queries, link, mbidtable)
	self:AtStartQuery(id, queries)
	self:SendMessage(id, itempoints, queries, link, mbidtable)
end

function mod:AtStartQuery(id, queries)
	self.MaxBid[id] = 0
	self.BidWarDone = nil
	self.master[id] = true
	if self.db.profile.silent then self.hasbid[id] = {} end
	if self:PlayerIncluded(queries) then self.master[id] = nil end
end

function mod:PlayerIncluded(dkp)
	for i, v in pairs(dkp) do
		if v[1].n == Player then
			return true
		end
	end
	return false
end

function mod:SendMessage(id, itempoints, data, link, mbidtable)
	self:SendCommMessage("MorgBid2", self:Serialize("BIDWAR", id, itempoints, data, db.quality, self.db.profile.bidstep, self.db.profile.overbid, self.db.profile.silent, link, db.mbidupdating, mbidtable), "RAID")
end

function mod:StartOutput(link, id, itempoints) 
	SendChatMessage(fmt(L["Beginning auction for %s:  ID = %s"], link, id), "RAID")
	SendChatMessage(fmt(L["Bid Format = mbid [ID/link] [%s/%s/%s] [bid value]"], db.NEED, db.TAKE, db.PASS), "RAID")
end

function mod:OnBidMessage(bid, sender, item, action)
	local POOL = self:CheckPool(item)
	local overbid = db.info[sender][POOL].points + self.db.profile.overbid
	if overbid < 0 then overbid = self.db.profile.overbid end
	if bid > overbid then
		bid = overbid
		SendChatMessage(fmt(L["You have exceeded the overbid amount of %s.  Your bid has been reset to the maximum."], self.db.profile.overbid), "WHISPER", nil, sender)
	end
	if mdkp.AuctionStatus[item] then
		if bid > self.MaxBid[item] then
			self.MaxBid[item] = bid
			self:BidWarQueryNeeded(item, sender, POOL, bid, nil, action)
		elseif bid == self.MaxBid[item] then
			self:BidWarQueryNeeded(item, sender, POOL, bid, true, action)
		end
	end
	return bid
end

function mod:OnOutputResluts(itemid, clickedwinner)
	SendChatMessage(L["Bidding is now closed."], "RAID")
	if self.db.profile.silent then self.hasbid[itemid] = nil end
end

function mod:BidWarQueryNeeded(id, bidder, POOL, bid, tied, action)
	if action == "OFFSPEC" then 
		for i,v in pairs(mdkp.querytooltips[id]) do
			if v[1].n == bidder then 
				v[3].n = 0 
				break
			end
		end
		return
	end
	
	db.items[id].points = bid
	if self.db.profile.silent then return end
	local _, Link, _, _, _, _, _, _,_, _ = GetItemInfo(id)
	if not tied then 
		SendChatMessage(fmt(L["New high bidder for %s: %s = %s"], Link, bidder, bid), "RAID")
	end
		
	--send new queries to needed members
	for i,v in pairs(mdkp.querytooltips[id]) do
		if v[1].n == bidder then 
			v[3].n = bid
			v[5] = db.info[v[1].n][POOL].points
			if tied then v[2] = "PENDING" end
		elseif v[2] == "NEED" or v[2] == "TAKE" then
			v[2] = "PENDING"
			v[5] = db.info[v[1].n][POOL].points
		end
	end
	local mbidtable = nil
	if db.mbidcustom then mbidtable = mdkp.statustext end
	self:SendCommMessage("MorgBid2", self:Serialize("BIDWAR", id, self.MaxBid[id] + self.db.profile.bidstep, mdkp.querytooltips[id], db.quality, self.db.profile.bidstep, self.db.profile.overbid, self.db.profile.silent, link, db.mbidupdating, mbidtable), "RAID")
end

function mod:GetItemValue(itemid, winner)
	local query = mdkp.querytooltips[itemid]
	local itempoints
	if self.db.profile.justenough then
		for num, data in ipairs(query) do
			if data[2] ~= "PASS" and data[2] ~= "PENDING" then
				if winner == data[1].n then
					if query[num + 1] and (query[num + 1][2] ~= "PASS" and query[num + 1][2] ~= "PENDING") then itempoints = query[num + 1][3].n
					else itempoints = 0 end
					if itempoints == 0 or itempoints > query[num][3].n then itempoints = query[num][3].n end
					break
				end
			end
		end
		mdkp.querytooltips[itemid][1][3].n = itempoints
	else 
		if winner ~= query[1][1].n then
			for num, data in pairs(query) do
				if data[1].n == winner then
					itempoints = query[num][3].n
					break
				end
			end
		else itempoints = query[1][3].n
		end
	end
	if not itempoints then itempoints = 0 end
	db.items[itemid].points = itempoints
	return itempoints, self.db.profile.silent
end

function mod:OnBidWhisper(sender, item, reply, bid)
	bid = tonumber(bid)
	if bid and bid > 0 then
		local POOL = self:CheckPool(item)
		for i,v in ipairs(mdkp.querytooltips[item]) do
			if v[1].n == sender then
				if self.db.profile.silent then
					if self.hasbid[item][sender] then
						SendChatMessage(L["Sorry this is a silent auction and you have already placed your bid."], "WHISPER", nil, sender)
						return
					else
						self.hasbid[item][sender] = true
					end
				end
				v[2] = reply
				local overbid = db.info[sender][POOL].points + self.db.profile.overbid
				if bid > overbid then
					bid = overbid
					SendChatMessage(fmt(L["You have exceeded the overbid amount of %s.  Your bid has been reset to the maximum."], self.db.profile.overbid), "WHISPER", nil, sender)
				end
				v[3].n = bid
				if bid > self.MaxBid[item] then
					self.MaxBid[item] = bid
					SendChatMessage(fmt(L["Your response has been accepted: %s"], bid), "WHISPER", nil, sender)
					self:BidWarQueryNeeded(item, sender, POOL, bid, nil, reply)
				elseif bid == self.MaxBid[item] then
					SendChatMessage(fmt(L["Your response has been accepted: %s"], bid), "WHISPER", nil, sender)
					self:BidWarQueryNeeded(item, sender, POOL, bid, true, reply)
				end
			end
		end
	end
end

function mod:ItemRecorded(itemId)
	db.items[itemId].points = 0
end
