local mod = MorgDKP2:NewModule("PointsDB")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")

local mdkp = MorgDKP2
mod.modName = "PointsDB"

local db
local band = bit.band
local bor = bit.bor
local bxor = bit.bxor
local fmt = string.format
local upper = string.upper
local dbversion = 1.3

local defaults = { 
	profile = {
		culllvl = 200, 
		memculllvl = 4,
		DBversion = nil,
   	},
}

StaticPopupDialogs["ImportMorgItems"] = {
	text = TEXT(L["Import default item database?"]),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		mod:ImportMorgItems()
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return nil
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("PointsDB", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	db.moduleON["PointsDB"] = true
	if not db.DBversion or dbversion > db.DBversion then self:ConvertDB() end
	if MorgDKP2_Items then StaticPopup_Show ("ImportMorgItems") end	--import item db?
	self:ImportDKPInfoFile()
end

function mod:OnDisable()
	db.moduleON["PointsDB"] = nil
end

function mod:UpdateDB()
	db = mdkp.db.profile
end

function mod:ConvertDB()
	local currtime = self:GetServerTime()
	local converted = 0
	local convtype = "1.3"
	db.Timestamp = time()
	
	if db.DBversion < 1.1 then
		for name, data in pairs(db.info) do
			db.info[name].lastupdate = currtime
			converted = converted + 1
		end
		convtype = convtype .. " 1.1"
		db.DBversion = 1.1
	end
	if db.DBversion < 1.2 then
		for name, _ in pairs(db.info) do
			if not db.info[name].raidloot or type(db.info[name].raidloot) ~= "table" then 
				db.info[name].raidloot = {} 
				converted = converted + 1
			end
		end
		convtype = convtype .. " 1.2"
		db.DBversion = 1.2
	end
	if db.DBversion < 1.3 then
		for name, data in pairs(db.info) do
			for field, data2 in pairs(data) do
				if field ~= "items" and field ~= "lastupdate" and field ~= "raidloot" and field ~= "aliases" and field ~= "class" then
					if data2.earned and not data2.attend then data2.attend = 0 end
				end
			end
		end
		db.DBversion = 1.3
	end
	if converted > 0 then
		self:out(fmt("Converted %s members to new format (%s).", converted, convtype))
	end
	--self:ClearRaidDB()
end

function mod:ImportMorgItems()
	local itemcount = self:ImportItemDB(MorgDKP2_Items)
	self:out(fmt(L["Database transfer complete. %s items were transferred and %s members updated."], itemcount, 0))
	self:out(L["Please delete the ItemData.lua file from the MorgDKP directory now."])
	MorgDKP2_Items = nil
end

function mod:ImportItemDB(items)
	local itemcount = 0
	if not db.items then db.items = {} end
	for id, data in pairs(items) do
		if not db.items[id] then
			db.items[id] = data
			itemcount = itemcount + 1
		end
	end
	return itemcount
end

function mod:ImportDKPInfoFile()
	local db = mdkp.db.profile
	local temptable = {}
	local count = 1
	if not MorgDKP2_DKP1 and not db.raid then self:DefaultDKP() end
	if not MorgDKP2_DKP1 then return end
	for i = 1,5 do
		local dkp = _G[("MorgDKP2_DKP" .. i)]
		if dkp then temptable[count] =  dkp
			count = count + 1
		end
	end
	local skipupdate = 1
	local timestamp
	if not db.Timestamp or type(temptable[1].Timestamp) == "string" or type(db.Timestamp) == "string" or temptable[1].Timestamp > db.Timestamp then
		skipupdate = nil
		db.Timestamp = temptable[1].Timestamp
		timestamp =  self:GetServerTime()
		self:out(L["Updating DKP Points.."])
	end
	if not skipupdate then
		db.eqDKP = { }
		db.raidlist = { }
		if not db.raid or db.raid == L["NONE"] or #temptable == 1 then db.raid = temptable[1].Pool end
		for i, dkpray in pairs(temptable) do
			if dkpray.Pool then
				db.raidlist[i] = dkpray.Pool
				db.eqDKP[dkpray.Pool] = { }
				db.eqDKP[dkpray.Pool]["eqDKPsite"] = dkpray.eqdkpsite
				db.eqDKP[dkpray.Pool]["prefix"] = dkpray.prefix
				if dkpray.Events then
					db.eqDKP[dkpray.Pool]["Events"] = { }
					for event, details in pairs(dkpray.Events) do
						db.eqDKP[dkpray.Pool]["Events"][event] = { }
						db.eqDKP[dkpray.Pool]["Events"][event] = { ["id"] = details.id, ["value"] = details.value}
					end
				end
				if dkpray.Ranks then
					local num = 1
					if not db.ranks[dkpray.Pool] then db.ranks[dkpray.Pool] = {} end
					for id, rank in pairs(dkpray.Ranks) do
						if not db.ranks[dkpray.Pool][rank] then db.ranks[dkpray.Pool][rank] = num end
						num = num + 1
					end
					db.useranks[dkpray.Pool] = true
				else
					db.useranks[dkpray.Pool] = nil
				end
				if dkpray.Aliases then
					db.eqDKP[dkpray.Pool]["Aliases"] = { }
					for alias, main in pairs(dkpray.Aliases) do
						db.eqDKP[dkpray.Pool]["Aliases"][alias] = main
						self:PlayerinDB(main)
						self:PlayerinDB(alias)
						local present = false
						if not db.info[main].aliases then db.info[main].aliases = { } end
						for numalias = 1 , #db.info[main].aliases do
							if db.info[main].aliases[numalias].alt == alias and db.info[main].aliases[numalias].raid == dkpray.Pool then
								present = true
							end
						end
						if not present then table.insert(db.info[main].aliases , {raid = dkpray.Pool, alt = alias}) end
					end
				end
			end
			if dkpray.Points then
				local count = 0
				for name, v in pairs(dkpray.Points) do
					if not db.info[name] then db.info[name] = { } end
					local earned, adjust, spent, balance = floor(v.earn * 100 + .5)/100, floor(v.adj * 100 + .5)/100, floor(v.spent * 100 + .5)/100, floor(v.bal * 100 + .5)/100
					local attend = (v.attend == 0 and db.newatt) or v.attend  
					db.info[name][dkpray.Pool] = {earned = earned + adjust, spent = spent, points = balance, attend = attend}
					if not db.info[name].class and v.class and v.class ~= "Unknown" then 
						local englishclass = mdkp.reverseclass[v.class]
						if englishclass then
							db.info[name].class = englishclass
						end
					end
					if not db.info[name].raidloot or type(db.info[name].raidloot) == "number" then db.info[name].raidloot = {} end
					db.info[name].lastupdate = timestamp
					if v.rank and v.rank ~= "NONE" then
						db.info[name].rank = v.rank
					end
					count = count + 1
					if db.info[name].aliases then
						for m = 1, #db.info[name].aliases do
							if db.info[name].aliases[m].raid == db.raid then
								local alt = db.info[name].aliases[m].alt
								local mainpoints = db.info[name][dkpray.Pool].points
								if not db.info[alt] then db.info[alt] = { } end
								db.info[alt][dkpray.Pool] = { 	points = mainpoints,
												earned = db.info[name][dkpray.Pool].earned,
												spent = db.info[name][dkpray.Pool].spent,
												attend = db.info[name][dkpray.Pool].attend}
								if not db.info[alt].raidloot or type(db.info[alt].raidloot) == "number" then db.info[alt].raidloot = {} end
								db.info[alt].lastupdate = timestamp
								self:debug(fmt(L["Added alias: %s of %s with %s DKP."], alt, name, mainpoints)) 
							end
						end
					end
				end
				self:out(fmt(L["Added %s players from %s"], count, dkpray.Pool))
			end
		end
		MorgDKP2_DKP1 = nil
		MorgDKP2_DKP2 = nil
		MorgDKP2_DKP3 = nil
		MorgDKP2_DKP4 = nil
		MorgDKP2_DKP5 = nil
	end
	self:ToggleFrameUpdate("TooltipLDB", true)
end

function mod:DefaultDKP()
	local db = mdkp.db.profile
	local name = UnitName("player")
	local ttime = time()
	MorgDKP2_DKP1 = {
		Timestamp = ttime,
		Pool = L["NONE"],
		eqdkpsite = "aqdkp",
		prefix = "eqdkp_",
		Points = {
			[name] = {earn = 1397.00,spent = 405.00,adj = 0.00,bal = 992}
		},
		Aliases = {
			["TestAlias"] = name
		},
		Events = {
			["Void Reaver"] = {id = 77,value = 3.00}
		},
	}
end
